pub struct AtomCollection<T: ModelInfo> { /* private fields */ }
Struct of Atom as data-driven design.

Implementations

source

impl<T: ModelInfo> AtomCollection<T>

source

pub fn update_symbol_at(
    &mut self,
    index: usize,
    new_symbol: &str
) -> Result<(), InvalidIndex>

Update the element_symbol at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn update_elm_id_at(
    &mut self,
    index: usize,
    new_elm_id: u8
) -> Result<(), InvalidIndex>

Update the element_id at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn update_xyz_at(
    &mut self,
    index: usize,
    new_xyz: Point3<f64>
) -> Result<(), InvalidIndex>

Update the xyz at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn update_frac_xyz_at(
    &mut self,
    index: usize,
    new_frac: Option<Point3<f64>>
) -> Result<(), InvalidIndex>

Update the fractional_xyz at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn update_atom_id_at(
    &mut self,
    index: usize,
    new_atom_id: u32
) -> Result<(), InvalidIndex>

Update the atom_id at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn update_atom_at(
    &mut self,
    index: usize,
    new_atom: Atom<T>
) -> Result<(), InvalidIndex>

Update the whole atom at the given index.

Errors

This function will return an error if the index is out of bounds.

source

pub fn element_symbols(&self) -> &[String]

source

pub fn atomic_nums(&self) -> &[u8]

source

pub fn xyz_coords(&self) -> &[Point3<f64>]

source

pub fn xyz_coords_mut(&mut self) -> &mut [Point3<f64>]

source

pub fn fractional_xyz(&self) -> &[Option<Point3<f64>>]

source

pub fn fractional_xyz_mut(&mut self) -> &mut [Option<Point3<f64>>]

source

pub fn atom_ids(&self) -> &[u32]

source

pub fn size(&self) -> usize