Struct castep_model_core::atom::Atom
source · pub struct Atom<T: ModelInfo> { /* private fields */ }Expand description
Struct that defines an atom.
Implementations
sourceimpl<T> Atom<T>where
T: ModelInfo,
impl<T> Atom<T>where
T: ModelInfo,
sourcepub fn new(
element_symbol: String,
element_id: u32,
xyz: Point3<f64>,
atom_id: u32,
model_type: T
) -> Self
pub fn new(
element_symbol: String,
element_id: u32,
xyz: Point3<f64>,
atom_id: u32,
model_type: T
) -> Self
Creates a new Atom.
sourcepub fn element_symbol(&self) -> &str
pub fn element_symbol(&self) -> &str
Returns a reference to the element symbol of this Atom<Format>.
sourcepub fn set_element_symbol(&mut self, element_symbol: String)
pub fn set_element_symbol(&mut self, element_symbol: String)
Sets the element symbol of this Atom<Format>.
sourcepub fn element_id(&self) -> u32
pub fn element_id(&self) -> u32
Returns the element id of this Atom<Format>.
sourcepub fn set_element_id(&mut self, element_id: u32)
pub fn set_element_id(&mut self, element_id: u32)
Sets the element id of this Atom<Format>.
sourcepub fn xyz(&self) -> &Point3<f64>
pub fn xyz(&self) -> &Point3<f64>
Returns a reference to the xyz of this Atom<Format>.
sourcepub fn set_xyz(&mut self, xyz: Point3<f64>)
pub fn set_xyz(&mut self, xyz: Point3<f64>)
Sets the xyz of this Atom<Format>.
sourcepub fn atom_id(&self) -> u32
pub fn atom_id(&self) -> u32
Returns the atom id of this Atom<Format>.
sourcepub fn set_atom_id(&mut self, atom_id: u32)
pub fn set_atom_id(&mut self, atom_id: u32)
Sets the atom id of this Atom<Format>.
sourcepub fn model_type(&self) -> &T
pub fn model_type(&self) -> &T
Returns a reference to the format of this Atom<Format>.
sourcepub fn set_model_type(&mut self, model_type: T)
pub fn set_model_type(&mut self, model_type: T)
Sets the format of this Atom<Format>.
pub fn fractional_xyz(&self) -> Option<&Point3<f64>>
pub fn set_fractional_xyz(&mut self, fractional_xyz: Option<Point3<f64>>)