pub struct AtomCollectionBuilder<T, S>where
    T: ModelInfo,
    S: BuildState,
{ /* private fields */ }

Implementations

source

impl<T: ModelInfo, S: BuildState> AtomCollectionBuilder<T, S>

source

pub fn new(size: usize) -> AtomCollectionBuilder<T, No>

source

pub fn with_element_symbols(
    self,
    element_symbols: &[String]
) -> Result<Self, AtomCollectionBuildingError>

Supply the element_symbols for an AtomCollection.

Errors

This function will return an error if the element_ids has a different vector size with the builder’s given size.

source

pub fn with_atomic_nums(
    self,
    atomic_nums: &[u8]
) -> Result<Self, AtomCollectionBuildingError>

Supply the atomic_nums for an AtomCollection.

Errors

This function will return an error if the element_ids has a different vector size with the builder’s given size.

source

pub fn with_xyz_coords(
    self,
    xyz_coords: &[Point3<f64>]
) -> Result<Self, AtomCollectionBuildingError>

Supply the xyz_coords for an AtomCollection.

Errors

This function will return an error if the xyz_coords has a different vector size with the builder’s given size.

source

pub fn with_fractional_xyz(
    self,
    fractional_xyz: &[Option<Point3<f64>>]
) -> Result<Self, AtomCollectionBuildingError>

Supply the fractional_xyz for an AtomCollection.

Errors

This function will return an error if the fractional_xyz has a different vector size with the builder’s given size.

source

pub fn with_atom_ids(
    self,
    atom_ids: &[u32]
) -> Result<Self, AtomCollectionBuildingError>

Supply the atom_ids for an AtomCollection.

Errors

This function will return an error if the atom_ids has a different vector size with the builder’s given size.

source

pub fn finish(
    self
) -> Result<AtomCollectionBuilder<T, Ready>, AtomCollectionBuildingError>

source

impl<T: ModelInfo> AtomCollectionBuilder<T, Ready>

source

pub fn build(self) -> AtomCollection<T>